// Datastore class to interact with a database.
class Datastore {
  // Name of the database connected to this Datastore.
  private _databaseName: string;
  // Name of the currently selected collection in the database.
  private selectedCollection: string = "";

  // Constructor to initialize the Datastore with a database name.
  constructor(databaseName: string) {
      this._databaseName = databaseName;
  }

  // Method to set the current collection on which operations are to be performed.
  collection(collectionName: string) {
      this.selectedCollection = collectionName;
      return this;
  }

  // Asynchronously retrieves a single document based on the provided query.
  // Optionally, a collection name can be specified; otherwise, the selected collection is used.
  async getOne(query: any, collectionName?: string): Promise<{ save: () => Promise<unknown>, remove: () => Promise<unknown> }> {
      // Set the collection if specified.
      if (collectionName) this.collection(collectionName);
      
      // Return a promise resolving to a mock data object.
      return new Promise((resolve, reject) => {
          let data = { save: () => Promise.resolve(), remove: () => Promise.resolve() };
          resolve(data);
      });
  }

  // Asynchronously retrieves multiple documents based on the provided query.
  // Optionally, a collection name can be specified; otherwise, the selected collection is used.
  async getMany(query: any, collectionName?: string): Promise<{ data: string }> {
      // Set the collection if specified.
      if (collectionName) this.collection(collectionName);
      
      // Return a promise resolving to mock data.
      return new Promise((resolve, reject) => {
          resolve({ data: "data" });
      });
  }

  // Asynchronously removes documents based on the provided query.
  // Optionally, a collection name can be specified; otherwise, the selected collection is used.
  async remove(query: any, collectionName?: string): Promise<{ data: string }> {
      // Set the collection if specified.
      if (collectionName) this.collection(collectionName);
      
      // Return a promise resolving to a success message.
      return new Promise((resolve, reject) => {
          resolve({ data: "data removed" });
      });
  }
}

// Export the Datastore class for use in other modules.
export default Datastore;
